<?php

/**
 * FileNotFound
 *
 * Deprecated: Warning, this class is deprecated and will be removed in a 
 * future version of SabreDAV. Please use Sabre_DAV_Exception_NotFound instead.
 *
 * @package Sabre
 * @subpackage DAV
 * @copyright Copyright (C) 2007-2013 Rooftop Solutions. All rights reserved.
 * @author Evert Pot (http://www.rooftopsolutions.nl/) 
 * @deprecated Use Sabre_DAV_Exception_NotFound instead
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
class Sabre_DAV_Exception_FileNotFound extends Sabre_DAV_Exception_NotFound {

}

