<?php

/**
 * This class contains the Sabre_CalDAV version constants.
 *
 * @package Sabre
 * @subpackage CalDAV 
 * @copyright Copyright (C) 2007-2013 Rooftop Solutions. All rights reserved.
 * @author Evert Pot (http://www.rooftopsolutions.nl/)
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
class Sabre_CalDAV_Version {

    /**
     * Full version number
     */
    const VERSION = '1.7.4';

    /**
     * Stability : alpha, beta, stable
     */
    const STABILITY = 'stable';

}
