<?php

// autoload_namespaces.php generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Sabre\\VObject' => $vendorDir . '/sabre/vobject/lib/',
    'Sabre' => $baseDir . '/lib/',
);
