<?php

/**
 * AddressBook interface
 *
 * Implement this interface to allow a node to be recognized as an addressbook.
 *
 * @package Sabre
 * @subpackage CardDAV
 * @copyright Copyright (C) 2007-2013 Rooftop Solutions. All rights reserved.
 * @author Evert Pot (http://www.rooftopsolutions.nl/) 
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
interface Sabre_CardDAV_IAddressBook extends Sabre_DAV_ICollection {



}
