<?php

namespace Sabre\VObject;

/**
 * Exception thrown by Reader if an invalid object was attempted to be parsed.
 *
 * @copyright Copyright (C) 2007-2013 Rooftop Solutions. All rights reserved.
 * @author Evert Pot (http://www.rooftopsolutions.nl/)
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
class ParseException extends \Exception { }
